const fs = require('fs')
const path = require('path')
const i18NPlugin = require('../src/index.js')
const pluginTester = require('babel-plugin-tester')

const pluginOptions = {
    language: 'de',
    inputPath: '__tests__/i18n',
    outputPath: '__tests__/i18n'
}

const languages = ['de', 'pt']

const mockLanguageFileContent = [
    {},
    {
        Hi: 'Hi// TODO',
        Hallo: 'Hallo// TODO',
        Hello: 'Hello// TODO'
    },
    {
        Hi: 'Hi// TODO',
        Hallo: 'Hallo// TODO'
    },
    {},
    {},
    {},
    {}
]

const mockJavascriptFileContent = [
    {
        code: 'var hello = __("Hello"); var hi = __("Hi"); var hallo = __("Hallo");',
        output: 'var hello = __("Hello");var hi = __("Hi");var hallo = __("Hallo");',
        parsedValue: {
            Hello: 'Hello// TODO',
            Hi: 'Hi// TODO',
            Hallo: 'Hallo// TODO'
        }
    },
    {
        code: 'var hello = __("Hello"); var hi = __("Hi"); var hallo = __("Hallo");',
        output: 'var hello = __("Hello");var hi = __("Hi");var hallo = __("Hallo");',
        parsedValue: {
            Hello: 'Hello// TODO',
            Hi: 'Hi// TODO',
            Hallo: 'Hallo// TODO'
        }
    },
    {
        code: 'var hello = __("Hello"); var newValue = __("newValue"); var hallo = __("Hallo");',
        output: 'var hello = __("Hello");var newValue = __("newValue");var hallo = __("Hallo");',
        parsedValue: {
            Hello: 'Hello// TODO',
            newValue: 'newValue// TODO',
            Hallo: 'Hallo// TODO'
        }
    },
    {
        code: 'var hello = __("Hello");',
        output: 'var hello = __("Hello");',
        parsedValue: {
            Hello: 'Hello// Perfect'
        }
    },
    {
        code: 'var hello = __("Ola");',
        output: 'var hello = __("Ola");',
        parsedValue: {
            Ola: 'Ola// TODO'
        }
    },
    {
        code: 'var hello = r__("Ola");',
        output: 'var hello = r__("Ola");',
        parsedValue: {
            Ola: 'Ola// TODO'
        }
    },
    {
        code: 'var hello = __("Ola");',
        output: 'var hello = "Ola// TODO";',
        parsedValue: {
            Ola: 'Ola// TODO'
        }
    }
]

function setupReadLanguageFile() {
    try {
        const fileLocation = `${process.cwd()}/${pluginOptions.inputPath}/${pluginOptions.language}.json`
        const fileContent = JSON.parse(fs.readFileSync(fileLocation))

        return fileContent
    } catch (error) {
        throw new Error('Not able to read the language file.')
    }
}

function writeToLanguageFile(content) {
    try {
        const outputLocation = `${process.cwd()}/${pluginOptions.inputPath}/${
            pluginOptions.language
        }.json`
        fs.writeFileSync(outputLocation, JSON.stringify(content, null, 4))
    } catch (err) {
        throw new Error('File should have been created')
    }
}

function verifyLanguageFile(content, languages = ['de']) {
    let fileContent = null

    languages.forEach(language => {
        try {
            const fileLocation = `${process.cwd()}/${pluginOptions.inputPath}/${language}.json`
            fileContent = JSON.parse(fs.readFileSync(fileLocation))
        } catch (error) {
            throw new Error('Not able to open and compare language file.')
        }
        if (fileContent) {
            Object.keys(fileContent).forEach(key => {
                const languageValue = content[key]
                expect(languageValue).toBe(fileContent[key])
            })
        }
    })
}

function resetLanguageFile(languages) {
    try {
        if (languages) {
            languages.forEach(language => {
                const outputLocation = `${process.cwd()}/${pluginOptions.inputPath}/${language}.json`
                fs.writeFileSync(outputLocation, '')
            })
        } else {
            const outputLocation = `${process.cwd()}/${pluginOptions.inputPath}/${
                pluginOptions.language
            }.json`
            fs.writeFileSync(outputLocation, '')
        }
    } catch (error) {}
}

pluginTester({
    plugin: i18NPlugin,
    pluginOptions,
    tests: [
        {
            title: 'Language File should be created if it does not exist',
            setup() {
                resetLanguageFile()
            },
            code: mockJavascriptFileContent[0].code,
            output: mockJavascriptFileContent[0].output,
            teardown() {
                expect(setupReadLanguageFile).toBeDefined()
                resetLanguageFile()
            }
        },
        {
            title: 'Language File should exists and keep the same content',
            setup() {
                resetLanguageFile()
                writeToLanguageFile(mockLanguageFileContent[1])
            },
            code: mockJavascriptFileContent[1].code,
            output: mockJavascriptFileContent[1].output,
            teardown() {
                verifyLanguageFile(mockJavascriptFileContent[1].parsedValue)
                resetLanguageFile()
            }
        },
        {
            title: 'Language File should exists and change the current content',
            setup() {
                resetLanguageFile()
                writeToLanguageFile(mockLanguageFileContent[2])
            },
            pluginOptions: {
                ...pluginOptions,
                test: true
            },
            code: mockJavascriptFileContent[2].code,
            output: mockJavascriptFileContent[2].output,
            teardown() {
                verifyLanguageFile(mockJavascriptFileContent[2].parsedValue)
                resetLanguageFile()
            }
        },
        {
            title: 'Language File should replace todoMessage',
            setup() {
                resetLanguageFile()
                writeToLanguageFile(mockLanguageFileContent[3])
            },
            pluginOptions: {
                ...pluginOptions,
                todoTag: '// Perfect',
                test: true
            },
            code: mockJavascriptFileContent[3].code,
            output: mockJavascriptFileContent[3].output,
            teardown() {
                verifyLanguageFile(mockJavascriptFileContent[3].parsedValue)
                resetLanguageFile()
            }
        },
        {
            title: 'Multiple language files generated',
            setup() {
                resetLanguageFile(languages)
                writeToLanguageFile(mockLanguageFileContent[4])
            },
            pluginOptions: {
                ...pluginOptions,
                language: languages,
                test: true
            },
            code: mockJavascriptFileContent[4].code,
            output: mockJavascriptFileContent[4].output,
            teardown() {
                verifyLanguageFile(mockJavascriptFileContent[4].parsedValue, languages)
                resetLanguageFile(languages)
            }
        },
        {
            title: 'Alias function replaced',
            setup() {
                resetLanguageFile(languages)
                writeToLanguageFile(mockLanguageFileContent[5])
            },
            pluginOptions: {
                ...pluginOptions,
                alias: 'r__'
            },
            code: mockJavascriptFileContent[5].code,
            output: mockJavascriptFileContent[5].output,
            teardown() {
                verifyLanguageFile(mockJavascriptFileContent[5].parsedValue)
                resetLanguageFile(languages)
            }
        },
        {
            title: 'Replace in file should replace string',
            setup() {
                resetLanguageFile(languages)
                writeToLanguageFile(mockLanguageFileContent[6])
            },
            pluginOptions: {
                ...pluginOptions,
                replaceInline: true
            },
            code: mockJavascriptFileContent[6].code,
            output: mockJavascriptFileContent[6].output,
            teardown() {
                verifyLanguageFile(mockJavascriptFileContent[6].parsedValue)
                resetLanguageFile(languages)
            }
        }
    ]
})
